/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* SCCS ID: @(#)devSend.c	1.19 11/14/93 */
/*************************************************************************
 * FILE:      devSend.c
 * AUTHOR:    Claude Saunders
 * Purpose:   
 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <rpc/types.h>
#include "libdev.h"

int main(int argc,char *argv[])
{
  void *val;
  void *pval;
  double stat[MAX_COMPONENTS];
 /* double atof();*/
  devnamearr mynames;
  devstructarr myinfo;
  int status;
  int i;
  int nels;
  int datatype;
  int dir;

  val = (void *)calloc(MAX_COMPONENTS*MAX_STRING_SIZE,sizeof(char));
  pval = val;

  if (argc < 2) {
    fprintf(stderr,"usage: devSend <device> [<msg>] [<val>]\n");
    exit(1);
  }

  /* Get various information about given device */
  status = devCount(argv[1], &nels, &mynames);
  if (devError(status))
    exit(1);

  status = devInfo(argv[1], &myinfo);
  if (devError(status))
    exit(1);

  /* If user supplies device name only, print info about device */
  if (argc == 2 ) {
    i=0;
    while(myinfo[i] != NULL) {
      printf("%-20s %-8s %s \n", 
	     myinfo[i]->dname,myinfo[i]->msg,myinfo[i]->pvname);
      i++;
    }
    exit(0);
  }

  devDirType(argv[2], myinfo, &dir, &datatype);

/*  
  if ( (dir == WRITE) && ((argc - 3) < nels) ) {
    fprintf(stderr,"devSend: you must supply %d arguments with this message\n",
	    nels);
    exit(1);
  }
*/

  /* Get any additional arguments for given device and message. */
  if (argc > 3)
    for (i=0 ; i < argc-3 ; i++) {
      switch (datatype) {
      case DBR_DOUBLE: {
	*((double *)pval) = atof(argv[i+3]);
	pval = (double *)pval + 1;
	break;
      }
      case DBR_STRING: {
	*(char **)pval = argv[i+3];
	pval = (char **)pval + 1;
	break;
      }
      }
    }

  devSend(argv[1],"status",stat);
  status = devSend(argv[1],argv[2],val);

  switch (dir) {
  case READ: {
    pval = val;
    for (i=0 ; i < nels ; i++) {
      if (status != DEV_NORMAL && status != DEV_NOMSG) 
	printf("%s NOCONNECT\n",mynames[i]);
      else if (!isnan(stat[i]) && stat[i] > 2.0) /* stat=invalid only */
	printf("%s BADSTATUS\n",mynames[i]);
      else
	switch (datatype) {
	case DBR_DOUBLE: {
	  if (isnan(*(double *)pval))
	    printf("%s NOHANDLE\n",mynames[i]);
	  else
	    printf("%s %le\n",mynames[i], *(double *)pval);
	  break;
	}
	case DBR_STRING: {
          if (*(char **)pval == NULL)
	    printf("%s NOHANDLE\n",mynames[i]);
          else
	    printf("%s %s\n",mynames[i],*(char **)pval);
	  break;
	}
	}
      switch (datatype) {
      case DBR_DOUBLE: {
	pval = (double *)pval + 1;
	break;
      }
      case DBR_STRING: {
	pval = (char **)pval + 1;
	break;
      }
      }
    }
    break;
  }
  case WRITE: {
    devError(status);
    break;
  }
  }
  ca_task_exit();
  return(0);
}
